"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const format = require("string-format");
const i18n_constants_1 = require("../i18n.constants");
const fs = require("fs");
const _ = require("lodash");
const path = require("path");
let I18nService = class I18nService {
    constructor(i18nOptions, translations, supportedLanguages, logger) {
        this.i18nOptions = i18nOptions;
        this.translations = translations;
        this.supportedLanguages = supportedLanguages;
        this.logger = logger;
    }
    translate(key, options) {
        options = Object.assign({ lang: this.i18nOptions.fallbackLanguage }, options);
        const { lang, args } = options;
        const translationsByLanguage = this.translations[lang];
        if (translationsByLanguage === undefined ||
            translationsByLanguage === null ||
            (!!translationsByLanguage && !translationsByLanguage.hasOwnProperty(key))) {
            if (this.i18nOptions.saveMissing === true) {
                this.saveMissingTranslation(key, lang);
            }
            if (lang !== this.i18nOptions.fallbackLanguage) {
                const message = `Translation "${key}" in "${lang}" does not exist.`;
                this.logger.error(message);
                return this.translate(key, {
                    lang: this.i18nOptions.fallbackLanguage,
                    args: args,
                });
            }
        }
        let translation = translationsByLanguage[key];
        if (translation && (args || (args instanceof Array && args.length > 0))) {
            translation = format(translation, ...(args instanceof Array ? args || [] : [args]));
        }
        return translation || key;
    }
    getSupportedLanguages() {
        return this.supportedLanguages;
    }
    saveMissingTranslation(key, language) {
        const filePathMissing = path.join(this.i18nOptions.path, language);
        if (!fs.existsSync(filePathMissing)) {
            this.logger.error(`Cannot find path to store missing translations`);
            return;
        }
        const keyParts = key.split('.');
        const filePart = keyParts.shift();
        const keyWithoutFile = keyParts.join('.');
        const filePath = path.join(filePathMissing, `${filePart}.missing`);
        let jsonContent = {};
        if (fs.existsSync(filePath)) {
            const fileContent = fs.readFileSync(filePath, { encoding: 'utf8' });
            jsonContent = JSON.parse(fileContent);
        }
        if (_.has(jsonContent, keyWithoutFile)) {
            return;
        }
        jsonContent = _.set(jsonContent, keyWithoutFile, '');
        fs.writeFileSync(filePath, JSON.stringify(jsonContent, null, 2));
        this.logger.error(`The key "${keyWithoutFile}" for language: ${language} was added to the file "${filePart}.missing" (@ ${filePath} )`);
    }
};
I18nService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(i18n_constants_1.I18N_OPTIONS)),
    __param(1, common_1.Inject(i18n_constants_1.I18N_TRANSLATIONS)),
    __param(2, common_1.Inject(i18n_constants_1.I18N_LANGUAGES)),
    __metadata("design:paramtypes", [Object, Object, Array, common_1.Logger])
], I18nService);
exports.I18nService = I18nService;
